#version 120

#define Bloom

//#define DOF
//#define MotionBlur

//#define AverageExposure

#define Brightness 1.00 //[0.70 0.72 0.74 0.76 0.78 0.80 0.82 0.84 0.86 0.88 0.90 0.92 0.94 0.96 0.98 1.00 1.02 1.04 1.06 1.08 1.10 1.12 1.14 1.16 1.18 1.20 1.22 1.24 1.26 1.28 1.30]
#define Saturation 1.00 //[0.70 0.72 0.74 0.76 0.78 0.80 0.82 0.84 0.86 0.88 0.90 0.92 0.94 0.96 0.98 1.00 1.02 1.04 1.06 1.08 1.10 1.12 1.14 1.16 1.18 1.20 1.22 1.24 1.26 1.28 1.30]
#define Contrast 1.00 //[0.70 0.72 0.74 0.76 0.78 0.80 0.82 0.84 0.86 0.88 0.90 0.92 0.94 0.96 0.98 1.00 1.02 1.04 1.06 1.08 1.10 1.12 1.14 1.16 1.18 1.20 1.22 1.24 1.26 1.28 1.30]

#define Weather
#define WeatherOpacity 1.00 //[0.25 0.50 0.75 1.00 1.25 1.50 1.75 2.00]

//#define WorldTimeAnimation

/*
FRAGDATA
0 raw solid
1 raw nonsolid
2 solid normals
3 nonsolid normals
4 solid matflag
5 nonsolid matflag
6 specular
7 rain

nonsolid : nonsolid block,water,hand

MATFLAG
.00 sky/sky
.10 none/water
.20 lava/none
.30 metal/none
.40 foliage/hand
.50 emissive/blockindicator
1.0 basic/basic
*/

#define MAX_COLOR_RANGE 32.0

varying vec4 texcoord;
varying float sunVisibility;
varying float moonVisibility;

uniform sampler2D colortex0;
uniform sampler2D colortex3;
uniform sampler2D colortex4;
uniform sampler2D colortex5;
#if defined DOF && defined Weather
uniform sampler2D colortex7;
#endif
uniform sampler2D depthtex0;
uniform sampler2D depthtex1;

uniform mat4 gbufferProjection;
uniform mat4 gbufferProjectionInverse;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferModelView;
#ifdef MotionBlur
uniform mat4 gbufferPreviousModelView;
uniform mat4 gbufferPreviousProjection;
#endif
uniform float rainStrength;
uniform ivec2 eyeBrightnessSmooth;
uniform float aspectRatio;
uniform float frameTimeCounter;
uniform float worldTime;
uniform int isEyeInWater;
uniform float centerDepthSmooth;
uniform vec3 cameraPosition;
uniform vec3 previousCameraPosition;
uniform vec3 sunPosition;
uniform float viewWidth;
uniform float viewHeight;

float eBS = eyeBrightnessSmooth.y/240.0;

#ifdef WorldTimeAnimation
float frametime = float(worldTime)/20.0;
#else
float frametime = frameTimeCounter;
#endif

vec2 getRefract(vec2 coord){
vec2 refract = vec2(cos(texcoord.y*32.0+frametime*4.0),sin(texcoord.x*32.0+frametime*4.0))*0.001;

vec2 newcoord = coord + refract*isEyeInWater;
float limit = float(newcoord.x < 0.0 || newcoord.x > 1.0 || newcoord.y < 0.0 || newcoord.y > 1.0);

return mix(newcoord,coord,limit);
}

vec3 Uncharted2Tonemap(vec3 x) {
	float A = 1.0;		//brightness multiplier
	float B = 0.37;		//black level (lower means darker and more constrasted, higher make the image whiter and less constrasted)
	float C = 0.1;		//constrast level 
	float D = 0.2;		
	float E = 0.02;
	float F = 0.3;
	return ((x*(A*x+C*B)+D*E)/(x*(A*x+B)+D*F))-E/F;
}

float getnoise(vec2 pos) {
return abs(fract(sin(dot(pos ,vec2(18.9898f,28.633f)+pos*100.0)) * 4378.5453f));
}

float dither4x4(vec2 pos)
{
	const int ditherPattern[16] = int[16](
		0, 8, 2, 10,
		12, 4, 14, 6,
		3, 11, 1, 9,
		15, 7, 13, 5);

    vec2 positon = floor(mod(vec2(texcoord.s * viewWidth,texcoord.t * viewHeight), 4.0f));

	int dither = ditherPattern[int(positon.x) + int(positon.y) * 4];

	return float(dither) / 16.0f;
}

#include "lib/fragPos.glsl"
#include "lib/lightColors.glsl"
#include "lib/lensFlare.glsl"

vec2 angleToVec2(float angle, float pi){
	return vec2(cos(angle*pi/180),sin(angle*pi/180));
}

void main() {
	const float pi = 22.0/7.0;
	vec2 newcoord = texcoord.xy;
	if (isEyeInWater > 0.9) newcoord = getRefract(texcoord.xy);
	vec3 color = pow(texture2D(colortex0, newcoord.st).rgb,vec3(2.2))*MAX_COLOR_RANGE;
	float matflag = texture2D(colortex4, texcoord.st).g;
	float wmatflag = texture2D(colortex5, texcoord.st).g;
	
	float land = float(matflag > 0.08);
	float hand = float(wmatflag > 0.38 && wmatflag < 0.42);
	float water = float(wmatflag > 0.08 && wmatflag < 0.12);
	
	vec3 blur = vec3(0.0);
	vec3 blur1 = vec3(0.0);
	
	#if defined Bloom || defined DOF
	vec2 halfpixel = vec2(1.0/viewWidth,1.0/viewHeight)*0.5;
	const float bpow = 0.25;
		 blur1 = pow(texture2D(colortex3,(newcoord - halfpixel)/pow(2.0,2.0) + vec2(0.0,0.0)).rgb,vec3(2.2))*pow(7.0,bpow);
	vec3 blur2 = pow(texture2D(colortex3,(newcoord - halfpixel)/pow(2.0,3.0) + vec2(0.3,0.0)).rgb,vec3(2.2))*pow(6.0,bpow);
	vec3 blur3 = pow(texture2D(colortex3,(newcoord - halfpixel)/pow(2.0,4.0) + vec2(0.0,0.3)).rgb,vec3(2.2))*pow(5.0,bpow);
	vec3 blur4 = pow(texture2D(colortex3,(newcoord - halfpixel)/pow(2.0,5.0) + vec2(0.1,0.3)).rgb,vec3(2.2))*pow(4.0,bpow);
	vec3 blur5 = pow(texture2D(colortex3,(newcoord - halfpixel)/pow(2.0,6.0) + vec2(0.2,0.3)).rgb,vec3(2.2))*pow(3.0,bpow);
	vec3 blur6 = pow(texture2D(colortex3,(newcoord - halfpixel)/pow(2.0,7.0) + vec2(0.3,0.3)).rgb,vec3(2.2))*pow(2.0,bpow);
	
	blur = (blur1 + blur2 + blur3 + blur4 + blur5 + blur6)*MAX_COLOR_RANGE;
	#endif
	
	//DoF
	float isdof = 0.0;
	#ifdef DOF
	float pixeldepth = texture2D(depthtex0,newcoord.xy).x;
	vec4 fragpos = getFragPos(newcoord,pixeldepth);
	vec4 worldpos = getWorldPos(fragpos);
	worldpos /= worldpos.w;
	
	float pcoc = 0;
	float fog = max((length(worldpos)-32.0)/1024.0,0.0);
	
	vec3 blurcol = clamp(mix(blur1*MAX_COLOR_RANGE/1.75,blur/8.606,rainStrength*pow(eBS,2.2))*0.44,vec3(0.0),vec3(2.2));
	float fbmix = 0.0;
	float nbmix = 0.0;
	
	fbmix = clamp(fog*8.0*mix(1.0+rainStrength,4.0,isEyeInWater),0.0,1.0)*(mix(land,1.0,isEyeInWater)*(1-rainStrength)+rainStrength);
	if (fbmix > 0.001) color = mix(color,blurcol,fbmix);

	float z = texture2D(depthtex1, newcoord.st).r;
	float focus = centerDepthSmooth;
	pcoc = clamp(abs(z-focus)/(6.0*(1.0+0.5*rainStrength*eBS))-0.1/1280,0,0.00625);
	//pcoc -= hand*0.0005;
	
	pcoc = pcoc*(1-hand);
	
	nbmix = pcoc/(0.00625);
	if (nbmix > 0.001) color = mix(color,blurcol,nbmix);
	
	#ifdef Weather
	vec4 rain = texture2D(colortex7,newcoord.xy);
	float raina = rain.a*pow(length(rain.rgb/3.0),0.5)*rainStrength*0.1*WeatherOpacity;
	if (hand < 0.9) color += rain.rgb*raina*(fbmix*(1.0-nbmix));
	#endif
	
	isdof = min(max(fbmix,nbmix)*2.0,1.0);
	#endif
	
	//Motion Blur
	#ifdef MotionBlur
	float motionblur  = texture2D(depthtex1, newcoord.st).x;
	vec3 mblur = pow(texture2D(colortex0,newcoord.st).rgb,vec3(2.2))*MAX_COLOR_RANGE;
	float mbwg = 1.0;
	float mbm = 0.0;
	vec2 pixel = 2.0/vec2(viewWidth,viewHeight);
	
	vec4 currentPosition = vec4(newcoord.x * 2.0 - 1.0, newcoord.y * 2.0 - 1.0, 2.0 * motionblur - 1.0, 1.0);
	
	vec4 fragposition = gbufferProjectionInverse * currentPosition;
	fragposition = gbufferModelViewInverse * fragposition;
	fragposition /= fragposition.w;
	fragposition.xyz += cameraPosition;
	
	vec4 previousPosition = fragposition;
	previousPosition.xyz -= previousCameraPosition;
	previousPosition = gbufferPreviousModelView * previousPosition;
	previousPosition = gbufferPreviousProjection * previousPosition;
	previousPosition /= previousPosition.w;

	vec2 velocity = (currentPosition - previousPosition).xy * 0.02;
	float velocitylimit = 0.01;
	velocity = normalize(velocity)*min(length(velocity),velocitylimit);
	
	vec2 coord = newcoord.st-velocity*4.0;
	for (int i = 0; i < 9; ++i, coord += velocity) {
		if (hand < 0.9){
			vec2 coordb = clamp(coord,pixel,1.0-pixel);
			vec3 temp = pow(texture2D(colortex0, coordb).rgb,vec3(2.2))*MAX_COLOR_RANGE;
			mblur += temp;
			mbwg += 1.0;
		}
	}
	mblur /= mbwg;
	mblur = mix(color,mblur,mbwg/9);

	color = mix(mblur,color,isdof);
	#endif
	
	//Bloom
	#ifdef Bloom
	color = mix(color,blur,0.02);
	#endif
	
	//Average Exposure
	#ifdef AverageExposure
	float ccol = min(length(pow(texture2DLod(colortex0, vec2(0.5),6.5).rgb,vec3(2.2))*MAX_COLOR_RANGE),1.0);
	color *= 1.0+(4.0-4.75*pow(ccol,0.15))*0.4;
	#endif
	
	//Tonemapping
	color = Uncharted2Tonemap(color)/Uncharted2Tonemap(vec3(1.0));
	
	//Vignette
	color *= 1.0-pow(length(texcoord.xy-0.5),2.0);
	
	//Final Color Adjustment
	color*= Brightness;
	color = color*Saturation-length(color/3)*(Saturation-1.0);
		
	color = pow(color,vec3(1.0/2.2));
	
	color = (color-0.5)*Contrast+0.5;
	
/* DRAWBUFFERS:0 */
	gl_FragData[0] = vec4(color,1.0);

}
